package org.openxma.dsl.generator.mojo;

import java.util.Collection;
import java.util.Properties;

import org.apache.maven.plugin.MojoExecutionException;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.mwe.core.WorkflowContext;
import org.eclipse.emf.mwe.core.WorkflowContextDefaultImpl;
import org.eclipse.emf.mwe.core.issues.IssuesImpl;
import org.eclipse.emf.mwe.core.monitor.NullProgressMonitor;
import org.eclipse.emf.mwe.utils.DirectoryCleaner;
import org.eclipse.xtext.mwe.NameBasedFilter;
import org.eclipse.xtext.mwe.Reader;
import org.eclipse.xtext.mwe.SlotEntry;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.dom.DomDslStandaloneSetup;
import org.openxma.dsl.generator.DefaultGeneratorConfiguration;
import org.openxma.dsl.generator.DefaultGeneratorFactory;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.component.DslReader;

/**
 * Maven plugin used to execute the openXMA Dom generator workflow
 * 
 * @goal generate-dom
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class DomGeneratorMojo extends AbstractGeneratorMojo {
    /**
     * The location of the custom template advice file.
     * 
     * @parameter
     */
    private String templateAdviceFile;

    public void doExecute() throws MojoExecutionException {
        extendCurrentClassloader(getClass().getClassLoader());
        Reader reader = new DslReader();
        reader.setUseJavaClassPath(true);
        DomDslStandaloneSetup setup = new DomDslStandaloneSetup();
        reader.addRegister(createCachedSetup(setup));
        reader.addPath(modelFolder);
        reader.setResourceSetProvider(createSharedResourceSet(setup));
        SlotEntry slotEntry = new SlotEntry();
        slotEntry.setSlot("model");
        slotEntry.setType(Model.class.getSimpleName());
        reader.addLoad(slotEntry);
        NameBasedFilter nameBasedFilter = new NameBasedFilter();
        nameBasedFilter.setRegularExpression("^(?:(?!classes|bin).)*\\.(dml|xmadsl)$");
        reader.setUriFilter(nameBasedFilter);
        WorkflowContext workflowContext = new WorkflowContextDefaultImpl();
        IssuesImpl issues = new IssuesImpl();
        NullProgressMonitor progressMonitor = new NullProgressMonitor();
        reader.invoke(workflowContext, progressMonitor, issues);

        if (cleanDirectory) {
            DirectoryCleaner directoryCleaner = new DirectoryCleaner();
            directoryCleaner.setDirectory(generatedSourceDirectory.toString());
            directoryCleaner.invoke(workflowContext, progressMonitor, issues);
            directoryCleaner.setDirectory(generatedResourceDirectory.toString());
            directoryCleaner.invoke(workflowContext, progressMonitor, issues);
        }

        DefaultGeneratorConfiguration generatorConfiguration = createGeneratorConfiguration(DslConstants.OPENXMA_DOM_FILE_EXTENSION);
        Generator generator = new DefaultGeneratorFactory().createAndInit(generatorConfiguration);
        @SuppressWarnings("unchecked")
        Collection<Model> models = (Collection<Model>) workflowContext.get("model");
        if (models != null) {
            for (Model model : models) {
                Resource resource = model.eResource();
                generator.generate(resource, progressMonitor);
            }
        }
    }

    @Override
    protected Properties loadProperties() throws MojoExecutionException {
        Properties properties = super.loadProperties();
        properties.setProperty(GeneratorConstants.XPAND_ASPECT_FILE, null != templateAdviceFile ? templateAdviceFile
                : "");
        return properties;
    }

}
